@@ -52,8 +52,9 @@ varnam_set_suggestions_dir (const char *dir);
 VARNAM_EXPORT extern int
 varnam_init(const char *scheme_file, varnam **handle, char **msg);
 
+
 VARNAM_EXPORT extern varray*
-varnam_get_all_scheme_details();
+varnam_get_all_handles();
 
 /**
  * Gets the scheme file currently used
@@ -303,15 +304,11 @@ varnam_set_scheme_details(
 
 /**
  * Gets the metadata for the current scheme
- * vscheme_details has to be freed by calling `varnam_destroy_scheme_details`
  *
  **/
 VARNAM_EXPORT extern int
 varnam_get_scheme_details(varnam *handle, vscheme_details **details);
 
-VARNAM_EXPORT extern void
-varnam_destroy_scheme_details(vscheme_details *details);
-
 /**
  * Returns error message for the most recent failed call to varnam API functions.
  *
