@@ -118,6 +118,8 @@ initialize_internal()
         vi->noMatchesCache = NULL;
         vi->tokenizationPossibility = NULL;
         vi->cached_stems = NULL;
+
+				vi->scheme_details = NULL;
     }
     return vi;
 }
@@ -469,8 +471,8 @@ scheme_details_new()
 	return details;
 }
 
-void
-varnam_destroy_scheme_details(vscheme_details *details)
+static void
+destroy_scheme_details(vscheme_details *details)
 {
 	if (details == NULL)
 		return;
@@ -484,13 +486,12 @@ varnam_destroy_scheme_details(vscheme_details *details)
 }
 
 varray*
-varnam_get_all_scheme_details()
+varnam_get_all_handles()
 {
 	tinydir_dir dir;
 	tinydir_file file;
-	varnam *handle;
-	vscheme_details *details;
-	varray *schemeDetails = NULL;
+	varnam *handle = NULL;
+	varray *handles = NULL;
 	int rc;
 	char *msg;
 
@@ -507,13 +508,8 @@ varnam_get_all_scheme_details()
 			if (!file.is_dir && (strcmp("vst", file.extension) == 0)) {
 				rc = varnam_init(file.path, &handle, &msg);
 				if (rc == VARNAM_SUCCESS) {
-					rc = varnam_get_scheme_details(handle, &details);
-					if (rc == VARNAM_SUCCESS) {
-						schemeDetails = schemeDetails == NULL ? varray_init() : schemeDetails;
-						varray_push(schemeDetails, details);
-						varnam_destroy(handle);
-						handle = NULL;
-					}
+					handles = handles == NULL ? varray_init() : handles;
+					varray_push(handles, handle);
 				}
 			}
 		}
@@ -521,21 +517,28 @@ varnam_get_all_scheme_details()
 	}
 
 	tinydir_close(&dir);
-	return schemeDetails;
+	return handles;
 }
 
 int
 varnam_get_scheme_details(varnam *handle, vscheme_details **details)
 {
 	int rc;
-	vscheme_details *d = scheme_details_new();
+	vscheme_details *d = NULL;
+
+	if (handle->internal->scheme_details != NULL) {
+		*details = handle->internal->scheme_details;
+		return VARNAM_SUCCESS;
+	}
+
+	d = scheme_details_new();
 	rc = vst_load_scheme_details(handle, d);
 	if (rc != VARNAM_SUCCESS) {
-		varnam_destroy_scheme_details(d);
 		return rc;
 	}
 
 	*details = d;
+	handle->internal->scheme_details = d;
 	return VARNAM_SUCCESS;
 }
 
@@ -922,5 +925,6 @@ destroy_varnam_internal(struct varnam_internal* vi)
     clear_cache (&vi->noMatchesCache);
     clear_cache (&vi->tokenizationPossibility);
     clear_cache (&vi->cached_stems);
+		destroy_scheme_details (vi->scheme_details);
     xfree(vi);
 }
